@@ -4,8 +4,10 @@
 import unittest
 import sys
 
+import itertools
+
 import bottle
-from bottle import request, tob, touni, tonat, json_dumps, _e, HTTPError, parse_date
+from bottle import request, tob, touni, tonat, json_dumps, HTTPError, parse_date, CookieError
 from . import tools
 import wsgiref.util
 import base64
@@ -13,17 +15,6 @@ import base64
 from bottle import BaseRequest, BaseResponse, LocalRequest
 
 
-try:
-    from itertools import product
-except ImportError:
-    def product(*args):
-        pools = map(tuple, args)
-        result = [[]]
-        for pool in pools:
-            result = [x + [y] for x in result for y in pool]
-        for prod in result:
-            yield tuple(prod)
-
 class TestRequest(unittest.TestCase):
 
     def test_app_property(self):
@@ -358,7 +349,6 @@ class TestRequest(unittest.TestCase):
         self.assertEqual('value1', request.POST['field1'])
         self.assertTrue('field1' not in request.files)
         self.assertEqual('value1', request.forms['field1'])
-        print(request.forms.dict, request.forms.recode_unicode)
         self.assertEqual('万难', request.forms['field2'])
         self.assertEqual(touni('万难'), request.forms.field2)
         # Field (multi)
@@ -468,11 +458,16 @@ class TestRequest(unittest.TestCase):
             self.assertEqual(r.foo, 'somevalue')
             self.assertTrue('somevalue' in r.environ.values())
 
+            # Attributes are read-only once set.
+            self.assertRaises(AttributeError, setattr, r, 'foo', 'x')
+
+            # Properties raise AttributeError.
+            self.assertRaises(AttributeError, setattr, r, 'body', 'x')
+
             # Unknown attributes raise AttributeError.
             self.assertRaises(AttributeError, getattr, r, 'somevalue')
 
 
-
 class TestResponse(unittest.TestCase):
 
     def test_constructor_body(self):
@@ -499,15 +494,24 @@ class TestResponse(unittest.TestCase):
         from functools import partial
         make_res = partial(BaseResponse, '', 200)
 
-        self.assertTrue('yay',
-            make_res([('x-test','yay')])['x-test'])
+        self.assertEqual('yay', make_res(x_test='yay')['x-test'])
 
-    def test_constructor_headerlist(self):
-        from functools import partial
-        make_res = partial(BaseResponse, '', 200)
-
-        self.assertTrue('yay', make_res(x_test='yay')['x-test'])
+    def test_wsgi_header_values(self):
+        def cmp(app, wire):
+            rs = BaseResponse()
+            rs.set_header('x-test', app)
+            result = [v for (h, v) in rs.headerlist if h.lower()=='x-test'][0]
+            self.assertEqual(wire, result)
 
+        if bottle.py3k:
+            cmp(1, tonat('1', 'latin1'))
+            cmp('öäü', 'öäü'.encode('utf8').decode('latin1'))
+            # Dropped byte header support in Python 3:
+            #cmp(tob('äöü'), 'äöü'.encode('utf8').decode('latin1'))
+        else:
+            cmp(1, '1')
+            cmp('öäü', 'öäü')
+            cmp(touni('äöü'), 'äöü')
 
     def test_set_status(self):
         rs = BaseResponse()
@@ -578,6 +582,14 @@ class TestResponse(unittest.TestCase):
         self.assertEqual(rs.status_code, 404) # last value
         self.assertEqual(rs.status_line, '404 Brain not Found') # last value
 
+        # Unicode in status line (thanks RFC7230 :/)
+        if bottle.py3k:
+            rs.status = '400 Non-ASÎÎ'
+            self.assertEqual(rs.status, rs.status_line)
+            self.assertEqual(rs.status_code, 400)
+            wire = rs._wsgi_status_line().encode('latin1')
+            self.assertEqual(rs.status, wire.decode('utf8'))
+
     def test_content_type(self):
         rs = BaseResponse()
         rs.content_type = 'test/some'
@@ -601,6 +613,14 @@ class TestResponse(unittest.TestCase):
         self.assertEqual(cookies[0], 'name1=value; Max-Age=5')
         self.assertEqual(cookies[1], 'name2="value 2"; Path=/foo')
 
+    def test_set_cookie_value_long_string(self):
+        r = BaseResponse()
+        self.assertRaises(ValueError, r.set_cookie, name='test', value='x' * 4097)
+
+    def test_set_cookie_name_long_string(self):
+        r = BaseResponse()
+        self.assertRaises(ValueError, r.set_cookie, name='x' * 4097, value='simple_value')
+
     def test_set_cookie_maxage(self):
         import datetime
         r = BaseResponse()
@@ -621,13 +641,57 @@ class TestResponse(unittest.TestCase):
         self.assertEqual(cookies[0], 'name1=value; expires=Thu, 01 Jan 1970 00:00:42 GMT')
         self.assertEqual(cookies[1], 'name2=value; expires=Thu, 01 Jan 1970 00:00:43 GMT')
 
+    def test_set_cookie_secure(self):
+        r = BaseResponse()
+        r.set_cookie('name1', 'value', secure=True)
+        r.set_cookie('name2', 'value', secure=False)
+        cookies = sorted([value for name, value in r.headerlist
+                   if name.title() == 'Set-Cookie'])
+        self.assertEqual(cookies[0].lower(), 'name1=value; secure')
+        self.assertEqual(cookies[1], 'name2=value')
+
+    def test_set_cookie_httponly(self):
+        if sys.version_info < (2,6,0):
+            return
+        r = BaseResponse()
+        r.set_cookie('name1', 'value', httponly=True)
+        r.set_cookie('name2', 'value', httponly=False)
+        cookies = sorted([value for name, value in r.headerlist
+                   if name.title() == 'Set-Cookie'])
+        self.assertEqual('name1=value; httponly', cookies[0].lower())
+        self.assertEqual('name2=value', cookies[1])
+
+    def test_set_cookie_samesite(self):
+        r = BaseResponse()
+        r.set_cookie('name1', 'value', same_site="lax")
+        r.set_cookie('name2', 'value', same_site="strict")
+
+        try:
+            r.set_cookie('name3', 'value', same_site='invalid')
+            self.fail("Should raise CookieError")
+        except CookieError:
+            pass
+
+        cookies = sorted([value for name, value in r.headerlist
+                   if name.title() == 'Set-Cookie'])
+        self.assertEqual('name1=value; samesite=lax', cookies[0].lower())
+        self.assertEqual('name2=value; samesite=strict', cookies[1].lower())
+
+    def test_clone_cookie(self):
+        r = BaseResponse()
+        r.set_cookie('name1', 'value', same_site="strict")
+        r2 = r.copy(BaseResponse)
+        cookies = sorted([value for name, value in r2.headerlist
+                   if name.title() == 'Set-Cookie'])
+        self.assertEqual('name1=value; samesite=strict', cookies[0].lower())
+
     def test_delete_cookie(self):
         response = BaseResponse()
         response.set_cookie('name', 'value')
         response.delete_cookie('name')
         cookies = [value for name, value in response.headerlist
                    if name.title() == 'Set-Cookie']
-        self.assertTrue('name=;' in cookies[0] or 'name="";' in cookies[0])
+        self.assertTrue('Max-Age=-1' in cookies[0])
 
     def test_set_header(self):
         response = BaseResponse()
@@ -669,7 +733,7 @@ class TestResponse(unittest.TestCase):
         response['x-test'] = 5
         self.assertEqual('5', response['x-test'])
         response['x-test'] = None
-        self.assertEqual('None', response['x-test'])
+        self.assertEqual('', response['x-test'])
         response['x-test'] = touni('瓶')
         self.assertEqual(tonat(touni('瓶')), response['x-test'])
 
@@ -679,7 +743,7 @@ class TestResponse(unittest.TestCase):
 
         # Test HeaderDict
         apis = 'append', 'replace', '__setitem__', 'setdefault'
-        for api, mask, test in product(apis, masks, tests):
+        for api, mask, test in itertools.product(apis, masks, tests):
             hd = bottle.HeaderDict()
             func = getattr(hd, api)
             value = mask.replace("{}", test)
@@ -688,7 +752,7 @@ class TestResponse(unittest.TestCase):
 
         # Test functions on BaseResponse
         apis = 'add_header', 'set_header', '__setitem__'
-        for api, mask, test in product(apis, masks, tests):
+        for api, mask, test in itertools.product(apis, masks, tests):
             rs = bottle.BaseResponse()
             func = getattr(rs, api)
             value = mask.replace("{}", test)
@@ -697,23 +761,25 @@ class TestResponse(unittest.TestCase):
 
     def test_expires_header(self):
         import datetime
+        from bottle import UTC
         response = BaseResponse()
-        now = datetime.datetime.now()
+        now = datetime.datetime.now(UTC)
         response.expires = now
-        
+
         def seconds(a, b):
             td = max(a,b) - min(a,b)
             return td.days*360*24 + td.seconds
-        
+
         self.assertEqual(0, seconds(response.expires, now))
-        now2 = datetime.datetime.utcfromtimestamp(
-            parse_date(response.headers['Expires']))
+        now2 = datetime.datetime.fromtimestamp(
+            parse_date(response.headers['Expires']), tz=UTC)
         self.assertEqual(0, seconds(now, now2))
 
+
 class TestRedirect(unittest.TestCase):
 
     def assertRedirect(self, target, result, query=None, status=303, **args):
-        env = {'SERVER_PROTOCOL':'HTTP/1.1'}
+        env = {'SERVER_PROTOCOL': 'HTTP/1.1'}
         for key in list(args):
             if key.startswith('wsgi'):
                 args[key.replace('_', '.', 1)] = args[key]
@@ -723,11 +789,10 @@ class TestRedirect(unittest.TestCase):
         bottle.response.bind()
         try:
             bottle.redirect(target, **(query or {}))
-        except bottle.HTTPResponse:
-            r = _e()
-            self.assertEqual(status, r.status_code)
-            self.assertTrue(r.headers)
-            self.assertEqual(result, r.headers['Location'])
+        except bottle.HTTPResponse as E:
+            self.assertEqual(status, E.status_code)
+            self.assertTrue(E.headers)
+            self.assertEqual(result, E.headers['Location'])
 
     def test_absolute_path(self):
         self.assertRedirect('/', 'http://127.0.0.1/')
@@ -814,8 +879,8 @@ class TestRedirect(unittest.TestCase):
         try:
             bottle.response.set_cookie('xxx', 'yyy')
             bottle.redirect('...')
-        except bottle.HTTPResponse:
-            h = [v for (k, v) in _e().headerlist if k == 'Set-Cookie']
+        except bottle.HTTPResponse as E:
+            h = [v for (k, v) in E.headerlist if k == 'Set-Cookie']
             self.assertEqual(h, ['xxx=yyy'])
 
 class TestWSGIHeaderDict(unittest.TestCase):
@@ -849,8 +914,3 @@ class TestWSGIHeaderDict(unittest.TestCase):
             self.assertTrue(key in self.headers)
             self.assertEqual(self.headers.get(key), 'test')
             self.assertEqual(self.headers.get(key, 5), 'test')
-
-
-
-if __name__ == '__main__': #pragma: no cover
-    unittest.main()
