@@ -2,6 +2,9 @@
 import unittest
 from . import tools
 
+from bottle import HTTPResponse, HTTPError, json_dumps
+
+
 class MyPlugin(object):
     def __init__(self):
         self.app = None
@@ -149,6 +152,16 @@ class TestPluginManagement(tools.ServerTestBase):
         self.assertBody('plugin;global-1', '/a')
         self.assertBody('plugin', '/b')
 
+    def test_json_plugin_catches_httpresponse(self):
+        @self.app.get('/return')
+        def _():
+            return HTTPResponse({'test': 'ko'}, 402)
+        @self.app.get('/raise')
+        def _():
+            raise HTTPResponse({'test': 'ko2'}, 402)
+
+        self.assertBody(json_dumps({'test': 'ko'}), '/return')
+        self.assertBody(json_dumps({'test': 'ko2'}), '/raise')
 
 
 class TestPluginAPI(tools.ServerTestBase):
@@ -170,9 +183,9 @@ class TestPluginAPI(tools.ServerTestBase):
 
     def test_apply(self):
         class Plugin(object):
-            def apply(self, func, cfg):
+            def apply(self, func, route):
                 def wrapper(*a, **ka):
-                    return func(test=cfg['config']['test'], *a, **ka) + '; tail'
+                    return func(test=route.config['test'], *a, **ka) + '; tail'
                 return wrapper
             def __call__(self, func):
                 raise AssertionError("Plugins must not be called "\
@@ -206,7 +219,3 @@ class TestPluginAPI(tools.ServerTestBase):
         self.assertTrue(getattr(plugin, 'closed', False))
         self.app.close()
         self.assertTrue(getattr(plugin2, 'closed', False))
-
-
-if __name__ == '__main__': #pragma: no cover
-    unittest.main()
